@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2009, 2010 by Daniel Stenberg
+ * Copyright (c) 2009-2011 by Daniel Stenberg
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms,
@@ -131,25 +131,22 @@ knownhost_add(LIBSSH2_KNOWNHOSTS *hosts,
               const char *comment, size_t commentlen,
               int typemask, struct libssh2_knownhost **store)
 {
-    struct known_host *entry =
-        LIBSSH2_ALLOC(hosts->session, sizeof(struct known_host));
+    struct known_host *entry;
     size_t hostlen = strlen(host);
     int rc;
     char *ptr;
     unsigned int ptrlen;
 
-    if(!entry)
+    /* make sure we have a key type set */
+    if(!(typemask & LIBSSH2_KNOWNHOST_KEY_MASK))
+        return _libssh2_error(hosts->session, LIBSSH2_ERROR_INVAL,
+                              "No key type set");
+
+    if(!(entry = LIBSSH2_ALLOC(hosts->session, sizeof(struct known_host))))
         return _libssh2_error(hosts->session, LIBSSH2_ERROR_ALLOC,
                               "Unable to allocate memory for known host "
                               "entry");
 
-    /* make sure we have a key type set */
-    if(!(typemask & LIBSSH2_KNOWNHOST_KEY_MASK)) {
-        rc = _libssh2_error(hosts->session, LIBSSH2_ERROR_INVAL,
-                            "No key type set");
-        goto error;
-    }
-
     memset(entry, 0, sizeof(struct known_host));
 
     entry->typemask = typemask;
@@ -420,7 +417,8 @@ knownhost_check(LIBSSH2_KNOWNHOSTS *hosts,
                 /* host name match, now compare the keys */
                 if(!strcmp(key, node->key)) {
                     /* they match! */
-                    *ext = knownhost_to_external(node);
+                    if (ext)
+                        *ext = knownhost_to_external(node);
                     badkey = NULL;
                     rc = LIBSSH2_KNOWNHOST_CHECK_MATCH;
                     break;
@@ -441,7 +439,8 @@ knownhost_check(LIBSSH2_KNOWNHOSTS *hosts,
 
     if(badkey) {
         /* key mismatch */
-        *ext = knownhost_to_external(badkey);
+        if (ext)
+            *ext = knownhost_to_external(badkey);
         rc = LIBSSH2_KNOWNHOST_CHECK_MISMATCH;
     }
 
@@ -617,7 +616,7 @@ static int oldstyle_hostline(LIBSSH2_KNOWNHOSTS *hosts,
 
             if(name > host) {
                 namelen = 0;
-                --name; // skip comma
+                --name; /* skip comma */
             }
         }
     }
