@@ -272,6 +272,8 @@ scp_recv(LIBSSH2_SESSION * session, const char *path, struct stat * sb)
 {
     int cmd_len;
     int rc;
+    int tmp_err_code;
+    const char *tmp_err_msg;
 
     if (session->scpRecv_state == libssh2_NB_state_idle) {
         session->scpRecv_mode = 0;
@@ -741,8 +743,12 @@ scp_recv(LIBSSH2_SESSION * session, const char *path, struct stat * sb)
         return session->scpRecv_channel;
     /* fall-through */
   scp_recv_error:
+    tmp_err_code = session->err_code;
+    tmp_err_msg = session->err_msg;
     while (libssh2_channel_free(session->scpRecv_channel) ==
            LIBSSH2_ERROR_EAGAIN);
+    session->err_code = tmp_err_code;
+    session->err_msg = tmp_err_msg;
     session->scpRecv_channel = NULL;
     session->scpRecv_state = libssh2_NB_state_idle;
     return NULL;
@@ -774,6 +780,8 @@ scp_send(LIBSSH2_SESSION * session, const char *path, int mode,
 {
     int cmd_len;
     int rc;
+    int tmp_err_code;
+    const char *tmp_err_msg;
 
     if (session->scpSend_state == libssh2_NB_state_idle) {
         session->scpSend_command_len =
@@ -881,7 +889,7 @@ scp_send(LIBSSH2_SESSION * session, const char *path, int mode,
             session->scpSend_response_len =
                 snprintf((char *) session->scpSend_response,
                          LIBSSH2_SCP_RESPONSE_BUFLEN, "T%ld 0 %ld 0\n",
-                         mtime, atime);
+                         (long)mtime, (long)atime);
             _libssh2_debug(session, LIBSSH2_TRACE_SCP, "Sent %s",
                            session->scpSend_response);
         }
@@ -1034,8 +1042,12 @@ scp_send(LIBSSH2_SESSION * session, const char *path, int mode,
         return session->scpSend_channel;
     /* fall-through */
   scp_send_error:
+    tmp_err_code = session->err_code;
+    tmp_err_msg = session->err_msg;
     while (libssh2_channel_free(session->scpSend_channel) ==
            LIBSSH2_ERROR_EAGAIN);
+    session->err_code = tmp_err_code;
+    session->err_msg = tmp_err_msg;
     session->scpSend_channel = NULL;
     session->scpSend_state = libssh2_NB_state_idle;
     return NULL;
