@@ -1,6 +1,6 @@
 /* Copyright (c) 2004-2007, Sara Golemon <sarag@libssh2.org>
  * Copyright (c) 2005 Mikhail Gusarov <dottedmag@dottedmag.net>
- * Copyright (c) 2009-2010 by Daniel Stenberg
+ * Copyright (c) 2009-2011 by Daniel Stenberg
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms,
@@ -665,14 +665,14 @@ userauth_hostbased_fromfile(LIBSSH2_SESSION *session,
         }
         else {
             /* Compute public key from private key. */
-            if (_libssh2_pub_priv_keyfile(session,
-                                          &session->userauth_host_method,
-                                          &session->userauth_host_method_len,
-                                          &pubkeydata, &pubkeydata_len,
-                                          privatekey, passphrase))
-                return _libssh2_error(session, LIBSSH2_ERROR_FILE,
-                                      "Unable to extract public key "
-                                      "from private key file");
+            rc = _libssh2_pub_priv_keyfile(session,
+                                           &session->userauth_host_method,
+                                           &session->userauth_host_method_len,
+                                           &pubkeydata, &pubkeydata_len,
+                                           privatekey, passphrase);
+            if (rc)
+                /* libssh2_pub_priv_keyfile calls _libssh2_error() */
+                return rc;
         }
 
         /*
@@ -1237,19 +1237,20 @@ userauth_publickey_fromfile(LIBSSH2_SESSION *session,
             rc = file_read_publickey(session, &session->userauth_pblc_method,
                                      &session->userauth_pblc_method_len,
                                      &pubkeydata, &pubkeydata_len,publickey);
-            if(rc)
+            if (rc)
                 return rc;
         }
         else {
             /* Compute public key from private key. */
-            if (_libssh2_pub_priv_keyfile(session,
-                                          &session->userauth_pblc_method,
-                                          &session->userauth_pblc_method_len,
-                                          &pubkeydata, &pubkeydata_len,
-                                          privatekey, passphrase))
-                return _libssh2_error(session, LIBSSH2_ERROR_FILE,
-                                      "Unable to extract public key "
-                                      "from private key file");
+            rc = _libssh2_pub_priv_keyfile(session,
+                                           &session->userauth_pblc_method,
+                                           &session->userauth_pblc_method_len,
+                                           &pubkeydata, &pubkeydata_len,
+                                           privatekey, passphrase);
+
+            /* _libssh2_pub_priv_keyfile calls _libssh2_error() */
+            if (rc)
+                return rc;
         }
     }
 
@@ -1447,44 +1448,50 @@ userauth_keyboard_interactive(LIBSSH2_SESSION * session,
             }
 
             /* server requested PAM-like conversation */
-
             s = session->userauth_kybd_data + 1;
 
             /* string    name (ISO-10646 UTF-8) */
             session->userauth_kybd_auth_name_len = _libssh2_ntohu32(s);
             s += 4;
-            session->userauth_kybd_auth_name =
-                LIBSSH2_ALLOC(session, session->userauth_kybd_auth_name_len);
-            if (!session->userauth_kybd_auth_name) {
-                _libssh2_error(session, LIBSSH2_ERROR_ALLOC,
-                               "Unable to allocate memory for "
-                               "keyboard-interactive 'name' request field");
-                goto cleanup;
+            if(session->userauth_kybd_auth_name_len) {
+                session->userauth_kybd_auth_name =
+                    LIBSSH2_ALLOC(session,
+                                  session->userauth_kybd_auth_name_len);
+                if (!session->userauth_kybd_auth_name) {
+                    _libssh2_error(session, LIBSSH2_ERROR_ALLOC,
+                                   "Unable to allocate memory for "
+                                   "keyboard-interactive 'name' "
+                                   "request field");
+                    goto cleanup;
+                }
+                memcpy(session->userauth_kybd_auth_name, s,
+                       session->userauth_kybd_auth_name_len);
+                s += session->userauth_kybd_auth_name_len;
             }
-            memcpy(session->userauth_kybd_auth_name, s,
-                   session->userauth_kybd_auth_name_len);
-            s += session->userauth_kybd_auth_name_len;
 
             /* string    instruction (ISO-10646 UTF-8) */
             session->userauth_kybd_auth_instruction_len = _libssh2_ntohu32(s);
             s += 4;
-            session->userauth_kybd_auth_instruction =
-                LIBSSH2_ALLOC(session,
-                              session->userauth_kybd_auth_instruction_len);
-            if (!session->userauth_kybd_auth_instruction) {
-                _libssh2_error(session, LIBSSH2_ERROR_ALLOC,
-                               "Unable to allocate memory for "
-                               "keyboard-interactive 'instruction' "
-                               "request field");
-                goto cleanup;
+            if(session->userauth_kybd_auth_instruction_len) {
+                session->userauth_kybd_auth_instruction =
+                    LIBSSH2_ALLOC(session,
+                                  session->userauth_kybd_auth_instruction_len);
+                if (!session->userauth_kybd_auth_instruction) {
+                    _libssh2_error(session, LIBSSH2_ERROR_ALLOC,
+                                   "Unable to allocate memory for "
+                                   "keyboard-interactive 'instruction' "
+                                   "request field");
+                    goto cleanup;
+                }
+                memcpy(session->userauth_kybd_auth_instruction, s,
+                       session->userauth_kybd_auth_instruction_len);
+                s += session->userauth_kybd_auth_instruction_len;
             }
-            memcpy(session->userauth_kybd_auth_instruction, s,
-                   session->userauth_kybd_auth_instruction_len);
-            s += session->userauth_kybd_auth_instruction_len;
 
             /* string    language tag (as defined in [RFC-3066]) */
             language_tag_len = _libssh2_ntohu32(s);
             s += 4;
+
             /* ignoring this field as deprecated */
             s += language_tag_len;
 
@@ -1492,53 +1499,56 @@ userauth_keyboard_interactive(LIBSSH2_SESSION * session,
             session->userauth_kybd_num_prompts = _libssh2_ntohu32(s);
             s += 4;
 
-            session->userauth_kybd_prompts =
-                LIBSSH2_ALLOC(session,
-                              sizeof(LIBSSH2_USERAUTH_KBDINT_PROMPT) *
-                              session->userauth_kybd_num_prompts);
-            if (!session->userauth_kybd_prompts) {
-                _libssh2_error(session, LIBSSH2_ERROR_ALLOC,
-                               "Unable to allocate memory for "
-                               "keyboard-interactive prompts array");
-                goto cleanup;
-            }
-            memset(session->userauth_kybd_prompts, 0,
-                   sizeof(LIBSSH2_USERAUTH_KBDINT_PROMPT) *
-                   session->userauth_kybd_num_prompts);
-
-            session->userauth_kybd_responses =
-                LIBSSH2_ALLOC(session,
-                              sizeof(LIBSSH2_USERAUTH_KBDINT_RESPONSE) *
-                              session->userauth_kybd_num_prompts);
-            if (!session->userauth_kybd_responses) {
-                _libssh2_error(session, LIBSSH2_ERROR_ALLOC,
-                               "Unable to allocate memory for "
-                               "keyboard-interactive responses array");
-                goto cleanup;
-            }
-            memset(session->userauth_kybd_responses, 0,
-                   sizeof(LIBSSH2_USERAUTH_KBDINT_RESPONSE) *
-                   session->userauth_kybd_num_prompts);
-
-            for(i = 0; i != session->userauth_kybd_num_prompts; ++i) {
-                /* string    prompt[1] (ISO-10646 UTF-8) */
-                session->userauth_kybd_prompts[i].length = _libssh2_ntohu32(s);
-                s += 4;
-                session->userauth_kybd_prompts[i].text =
+            if(session->userauth_kybd_num_prompts) {
+                session->userauth_kybd_prompts =
                     LIBSSH2_ALLOC(session,
-                                  session->userauth_kybd_prompts[i].length);
-                if (!session->userauth_kybd_prompts[i].text) {
+                                  sizeof(LIBSSH2_USERAUTH_KBDINT_PROMPT) *
+                                  session->userauth_kybd_num_prompts);
+                if (!session->userauth_kybd_prompts) {
                     _libssh2_error(session, LIBSSH2_ERROR_ALLOC,
                                    "Unable to allocate memory for "
-                                   "keyboard-interactive prompt message");
+                                   "keyboard-interactive prompts array");
                     goto cleanup;
                 }
-                memcpy(session->userauth_kybd_prompts[i].text, s,
-                       session->userauth_kybd_prompts[i].length);
-                s += session->userauth_kybd_prompts[i].length;
+                memset(session->userauth_kybd_prompts, 0,
+                       sizeof(LIBSSH2_USERAUTH_KBDINT_PROMPT) *
+                       session->userauth_kybd_num_prompts);
 
-                /* boolean   echo[1] */
-                session->userauth_kybd_prompts[i].echo = *s++;
+                session->userauth_kybd_responses =
+                    LIBSSH2_ALLOC(session,
+                                  sizeof(LIBSSH2_USERAUTH_KBDINT_RESPONSE) *
+                                  session->userauth_kybd_num_prompts);
+                if (!session->userauth_kybd_responses) {
+                    _libssh2_error(session, LIBSSH2_ERROR_ALLOC,
+                                   "Unable to allocate memory for "
+                                   "keyboard-interactive responses array");
+                    goto cleanup;
+                }
+                memset(session->userauth_kybd_responses, 0,
+                       sizeof(LIBSSH2_USERAUTH_KBDINT_RESPONSE) *
+                       session->userauth_kybd_num_prompts);
+
+                for(i = 0; i != session->userauth_kybd_num_prompts; ++i) {
+                    /* string    prompt[1] (ISO-10646 UTF-8) */
+                    session->userauth_kybd_prompts[i].length =
+                        _libssh2_ntohu32(s);
+                    s += 4;
+                    session->userauth_kybd_prompts[i].text =
+                        LIBSSH2_ALLOC(session,
+                                      session->userauth_kybd_prompts[i].length);
+                    if (!session->userauth_kybd_prompts[i].text) {
+                        _libssh2_error(session, LIBSSH2_ERROR_ALLOC,
+                                       "Unable to allocate memory for "
+                                       "keyboard-interactive prompt message");
+                        goto cleanup;
+                    }
+                    memcpy(session->userauth_kybd_prompts[i].text, s,
+                           session->userauth_kybd_prompts[i].length);
+                    s += session->userauth_kybd_prompts[i].length;
+
+                    /* boolean   echo[1] */
+                    session->userauth_kybd_prompts[i].echo = *s++;
+                }
             }
 
             response_callback(session->userauth_kybd_auth_name,
@@ -1595,10 +1605,9 @@ userauth_keyboard_interactive(LIBSSH2_SESSION * session,
             rc = _libssh2_transport_send(session, session->userauth_kybd_data,
                                          session->userauth_kybd_packet_len,
                                          NULL, 0);
-            if (rc == LIBSSH2_ERROR_EAGAIN) {
+            if (rc == LIBSSH2_ERROR_EAGAIN)
                 return _libssh2_error(session, LIBSSH2_ERROR_EAGAIN,
                                       "Would block");
-            }
             if (rc) {
                 _libssh2_error(session, LIBSSH2_ERROR_SOCKET_SEND,
                                "Unable to send userauth-keyboard-interactive"
@@ -1633,14 +1642,22 @@ userauth_keyboard_interactive(LIBSSH2_SESSION * session,
             }
         }
 
-        LIBSSH2_FREE(session, session->userauth_kybd_prompts);
-        session->userauth_kybd_prompts = NULL;
-        LIBSSH2_FREE(session, session->userauth_kybd_responses);
-        session->userauth_kybd_responses = NULL;
-        LIBSSH2_FREE(session, session->userauth_kybd_auth_name);
-        session->userauth_kybd_auth_name = NULL;
-        LIBSSH2_FREE(session, session->userauth_kybd_auth_instruction);
-        session->userauth_kybd_auth_instruction = NULL;
+        if(session->userauth_kybd_prompts) {
+            LIBSSH2_FREE(session, session->userauth_kybd_prompts);
+            session->userauth_kybd_prompts = NULL;
+        }
+        if(session->userauth_kybd_responses) {
+            LIBSSH2_FREE(session, session->userauth_kybd_responses);
+            session->userauth_kybd_responses = NULL;
+        }
+        if(session->userauth_kybd_auth_name) {
+            LIBSSH2_FREE(session, session->userauth_kybd_auth_name);
+            session->userauth_kybd_auth_name = NULL;
+        }
+        if(session->userauth_kybd_auth_instruction) {
+            LIBSSH2_FREE(session, session->userauth_kybd_auth_instruction);
+            session->userauth_kybd_auth_instruction = NULL;
+        }
 
         if (session->userauth_kybd_auth_failure) {
             session->userauth_kybd_state = libssh2_NB_state_idle;
